"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const BlinkDiff = require("blink-diff");
const PngJsImage = require("pngjs-image");
const image_options_1 = require("./image-options");
const log_image_type_1 = require("./enums/log-image-type");
const utils_1 = require("./utils");
class ImageHelper {
    constructor(_args, _driver) {
        this._args = _args;
        this._driver = _driver;
        this._imagesResults = new Map();
        this._options = {
            timeOutSeconds: 2,
            tolerance: 0,
            toleranceType: image_options_1.ImageOptions.pixel,
            waitOnCreatingInitialSnapshot: 2000,
            preserveImageName: false,
        };
    }
    get options() {
        return this._options;
    }
    set options(options) {
        this._options = this.extendOptions(options);
    }
    set testName(testName) {
        this._testName = testName;
    }
    get testName() {
        return this._testName;
    }
    get imageComppareOptions() {
        this.extendOptions(this._options);
        return this._options;
    }
    set imageComppareOptions(imageComppareOptions) {
        this._options = this.extendOptions(imageComppareOptions);
    }
    compareScreen(options) {
        return __awaiter(this, void 0, void 0, function* () {
            options = this.extendOptions(options);
            const imageName = this.increaseImageName(options.imageName || this._testName);
            const result = yield this._driver.compareScreen(imageName, options.timeOutSeconds, options.tolerance, options.toleranceType);
            this._imagesResults.set(imageName, result);
            return result;
        });
    }
    compareElement(element, options) {
        return __awaiter(this, void 0, void 0, function* () {
            options = this.extendOptions(options);
            const imageName = this.increaseImageName(options.imageName || this._testName);
            const result = yield this._driver.compareElement(element, imageName, options.tolerance, options.timeOutSeconds, options.toleranceType);
            this._imagesResults.set(imageName, result);
            return result;
        });
    }
    compareRectangle(element, options) {
        return __awaiter(this, void 0, void 0, function* () {
            options = this.extendOptions(options);
            const imageName = this.increaseImageName(options.imageName || this._testName);
            const result = yield this._driver.compareRectangle(element, imageName, options.timeOutSeconds, options.tolerance, options.toleranceType);
            this._imagesResults.set(imageName, result);
            return result;
        });
    }
    hasImageComparisonPassed() {
        let shouldFailTest = true;
        console.log();
        this._imagesResults.forEach((v, k, map) => {
            if (!this._imagesResults.get(k)) {
                shouldFailTest = false;
                this._driver.testReporterLog(`Image comparison for image ${k} has failed!`);
                utils_1.logError(`Image comparison for image ${k} has failed`);
            }
        });
        this.reset();
        return shouldFailTest;
    }
    reset() {
        this._imagesResults.clear();
    }
    increaseImageName(imageName) {
        if (this._imagesResults.size > 1) {
            const number = /\d+$/.test(imageName) ? +`${/\d+$/.exec(imageName)}` + 1 : `2`;
            imageName = `${imageName}_${number}`;
        }
        return imageName;
    }
    extendOptions(options) {
        options = options || {};
        Object.getOwnPropertyNames(this.options).forEach(prop => {
            if (!options[prop]) {
                options[prop] = this.options[prop];
            }
        });
        return options;
    }
    get imageCropRect() {
        return this._imageCropRect;
    }
    set imageCropRect(rect) {
        this._imageCropRect = rect;
    }
    get blockOutAreas() {
        return this._blockOutAreas;
    }
    set blockOutAreas(rectangles) {
        this._blockOutAreas = rectangles;
    }
    imageOutputLimit() {
        return image_options_1.ImageOptions.outputAll;
    }
    thresholdType() {
        return image_options_1.ImageOptions.percent;
    }
    threshold(thresholdType) {
        if (thresholdType == image_options_1.ImageOptions.percent) {
            return 0.01; // 0.01 = 1 percent; 500 percent
        }
        else {
            return 10; // 10 pixels
        }
    }
    delta() {
        return 20;
    }
    static cropImageDefault(_args) {
        return { x: 0, y: ImageHelper.getOffsetPixels(_args), width: undefined, height: undefined };
    }
    static getOffsetPixels(args) {
        return args.device.config ? args.device.config.offsetPixels : 0;
    }
    runDiff(diffOptions, diffImage) {
        var that = this;
        return new Promise((resolve, reject) => {
            diffOptions.run(function (error, result) {
                if (error) {
                    throw error;
                }
                else {
                    let message;
                    let resultCode = diffOptions.hasPassed(result.code);
                    if (resultCode) {
                        message = "Screen compare passed!";
                        console.log(message);
                        console.log('Found ' + result.differences + ' differences.');
                        return resolve(true);
                    }
                    else {
                        message = `Screen compare failed! Found ${result.differences} differences.\n`;
                        console.log(message);
                        if (Object.getOwnPropertyNames(that._args.testReporter).length > 0 && that._args.testReporter.logImageTypes && that._args.testReporter.logImageTypes.indexOf(log_image_type_1.LogImageType.everyImage) > -1) {
                            that._args.testReporterLog(`${path_1.basename(diffImage)}\n\r${message}`);
                            that._args.testReporterLog(diffImage);
                        }
                        return resolve(false);
                    }
                }
            });
        });
    }
    compareImages(actual, expected, output, valueThreshold = this.threshold(this.thresholdType()), typeThreshold = this.thresholdType()) {
        const diff = new BlinkDiff({
            imageAPath: actual,
            imageBPath: expected,
            imageOutputPath: output,
            imageOutputLimit: this.imageOutputLimit(),
            thresholdType: typeThreshold,
            threshold: valueThreshold,
            delta: this.delta(),
            cropImageA: this._imageCropRect,
            cropImageB: this._imageCropRect,
            blockOut: this._blockOutAreas,
            verbose: this._args.verbose,
        });
        if (typeThreshold == image_options_1.ImageOptions.percent) {
            valueThreshold = Math.floor(valueThreshold * 100);
            console.log(`Using ${valueThreshold}\ ${typeThreshold} tolerance`);
        }
        else {
            console.log(`Using ${valueThreshold} tolerance`);
        }
        const result = this.runDiff(diff, output);
        this._blockOutAreas = undefined;
        return result;
    }
    clipRectangleImage(rect, path) {
        return __awaiter(this, void 0, void 0, function* () {
            let imageToClip;
            imageToClip = yield this.readImage(path);
            imageToClip.clip(rect.x, rect.y, rect.width, rect.height);
            return new Promise((resolve, reject) => {
                imageToClip.writeImage(path, (err) => {
                    if (err) {
                        return reject(err);
                    }
                    return resolve();
                });
            });
        });
    }
    readImage(path) {
        return new Promise((resolve, reject) => {
            PngJsImage.readImage(path, (err, image) => {
                if (err) {
                    return reject(err);
                }
                return resolve(image);
            });
        });
    }
}
exports.ImageHelper = ImageHelper;
//# sourceMappingURL=image-helper.js.map